"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelApi = void 0;
const Finder_1 = require("./Finder");
const clock_1 = require("../../../json-crdt-patch/clock");
const NoopOperation_1 = require("../../../json-crdt-patch/operations/NoopOperation");
const constants_1 = require("../../constants");
const Patch_1 = require("../../../json-crdt-patch/Patch");
const PatchBuilder_1 = require("../../../json-crdt-patch/PatchBuilder");
class ModelApi {
    constructor(model) {
        this.model = model;
        this.patches = [];
        this.builder = new PatchBuilder_1.PatchBuilder(model.clock);
    }
    getApi() {
        return this;
    }
    getNode() {
        const model = this.model;
        const id = model.root.toValue();
        return model.node(id) || constants_1.NULL;
    }
    find(path) {
        const finder = new Finder_1.Finder(this.getNode(), this);
        return path ? finder.find(path) : finder;
    }
    val(path) {
        return this.find().val(path);
    }
    str(path) {
        return this.find().str(path);
    }
    bin(path) {
        return this.find().bin(path);
    }
    arr(path) {
        return this.find().arr(path);
    }
    obj(path) {
        return this.find().obj(path);
    }
    commit() {
        const patch = this.builder.patch;
        this.builder = new PatchBuilder_1.PatchBuilder(this.model.clock);
        if (patch.ops.length) {
            this.patches.push(patch);
            this.model.applyPatch(patch);
        }
        return patch;
    }
    flush() {
        const patches = this.patches;
        this.patches = [];
        return patches;
    }
    flushPatch() {
        const patches = this.flush();
        const length = patches.length;
        const result = new Patch_1.Patch();
        let prev = null;
        for (let i = 0; i < length; i++) {
            const patch = patches[i];
            if (!patch.ops)
                continue;
            if (!prev)
                result.ops.push(...patch.ops);
            else {
                const id = patch.getId();
                const nextTime = prev.nextTime();
                const gap = id.time - nextTime;
                if (gap > 0)
                    result.ops.push(new NoopOperation_1.NoopOperation(new clock_1.LogicalTimestamp(id.getSessionId(), nextTime), gap));
                result.ops.push(...patch.ops);
            }
            prev = patch;
        }
        return result;
    }
    patch(cb) {
        this.commit();
        cb(this);
        this.commit();
    }
    root(json) {
        const value = this.builder.json(json);
        this.builder.root(value);
        return this;
    }
    toView() {
        return this.getNode().toJson();
    }
}
exports.ModelApi = ModelApi;
