import { Finder } from './Finder';
import { LogicalTimestamp } from '../../../json-crdt-patch/clock';
import { NoopOperation } from '../../../json-crdt-patch/operations/NoopOperation';
import { NULL } from '../../constants';
import { Patch } from '../../../json-crdt-patch/Patch';
import { PatchBuilder } from '../../../json-crdt-patch/PatchBuilder';
export class ModelApi {
    model;
    patches = [];
    builder;
    getApi() {
        return this;
    }
    constructor(model) {
        this.model = model;
        this.builder = new PatchBuilder(model.clock);
    }
    getNode() {
        const model = this.model;
        const id = model.root.toValue();
        return model.node(id) || NULL;
    }
    find(path) {
        const finder = new Finder(this.getNode(), this);
        return path ? finder.find(path) : finder;
    }
    val(path) {
        return this.find().val(path);
    }
    str(path) {
        return this.find().str(path);
    }
    bin(path) {
        return this.find().bin(path);
    }
    arr(path) {
        return this.find().arr(path);
    }
    obj(path) {
        return this.find().obj(path);
    }
    commit() {
        const patch = this.builder.patch;
        this.builder = new PatchBuilder(this.model.clock);
        if (patch.ops.length) {
            this.patches.push(patch);
            this.model.applyPatch(patch);
        }
        return patch;
    }
    flush() {
        const patches = this.patches;
        this.patches = [];
        return patches;
    }
    flushPatch() {
        const patches = this.flush();
        const length = patches.length;
        const result = new Patch();
        let prev = null;
        for (let i = 0; i < length; i++) {
            const patch = patches[i];
            if (!patch.ops)
                continue;
            if (!prev)
                result.ops.push(...patch.ops);
            else {
                const id = patch.getId();
                const nextTime = prev.nextTime();
                const gap = id.time - nextTime;
                if (gap > 0)
                    result.ops.push(new NoopOperation(new LogicalTimestamp(id.getSessionId(), nextTime), gap));
                result.ops.push(...patch.ops);
            }
            prev = patch;
        }
        return result;
    }
    patch(cb) {
        this.commit();
        cb(this);
        this.commit();
    }
    root(json) {
        const value = this.builder.json(json);
        this.builder.root(value);
        return this;
    }
    toView() {
        return this.getNode().toJson();
    }
}
