"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Draft = void 0;
var tslib_1 = require("tslib");
var DeleteOperation_1 = require("./operations/DeleteOperation");
var InsertArrayElementsOperation_1 = require("./operations/InsertArrayElementsOperation");
var InsertBinaryDataOperation_1 = require("./operations/InsertBinaryDataOperation");
var InsertStringSubstringOperation_1 = require("./operations/InsertStringSubstringOperation");
var clock_1 = require("./clock");
var MakeArrayOperation_1 = require("./operations/MakeArrayOperation");
var MakeBinaryOperation_1 = require("./operations/MakeBinaryOperation");
var MakeConstantOperation_1 = require("./operations/MakeConstantOperation");
var MakeNumberOperation_1 = require("./operations/MakeNumberOperation");
var MakeObjectOperation_1 = require("./operations/MakeObjectOperation");
var MakeStringOperation_1 = require("./operations/MakeStringOperation");
var MakeValueOperation_1 = require("./operations/MakeValueOperation");
var NoopOperation_1 = require("./operations/NoopOperation");
var Patch_1 = require("./Patch");
var PatchBuilder_1 = require("./PatchBuilder");
var SetNumberOperation_1 = require("./operations/SetNumberOperation");
var SetObjectKeysOperation_1 = require("./operations/SetObjectKeysOperation");
var SetRootOperation_1 = require("./operations/SetRootOperation");
var Draft = (function () {
    function Draft() {
        this.builder = new PatchBuilder_1.PatchBuilder(new clock_1.LogicalVectorClock(-1, 0));
    }
    Draft.prototype.patch = function (clock) {
        var e_1, _a;
        var patch = new Patch_1.Patch();
        var ops = this.builder.patch.ops;
        var ts = function (id) {
            return id.getSessionId() !== -1 ? id : new clock_1.LogicalTimestamp(clock.getSessionId(), clock.time + id.time);
        };
        try {
            for (var ops_1 = (0, tslib_1.__values)(ops), ops_1_1 = ops_1.next(); !ops_1_1.done; ops_1_1 = ops_1.next()) {
                var op = ops_1_1.value;
                if (op instanceof DeleteOperation_1.DeleteOperation)
                    patch.ops.push(new DeleteOperation_1.DeleteOperation(ts(op.id), ts(op.obj), ts(op.after), op.length));
                else if (op instanceof InsertArrayElementsOperation_1.InsertArrayElementsOperation)
                    patch.ops.push(new InsertArrayElementsOperation_1.InsertArrayElementsOperation(ts(op.id), ts(op.arr), ts(op.after), op.elements.map(ts)));
                else if (op instanceof InsertStringSubstringOperation_1.InsertStringSubstringOperation)
                    patch.ops.push(new InsertStringSubstringOperation_1.InsertStringSubstringOperation(ts(op.id), ts(op.obj), ts(op.after), op.substring));
                else if (op instanceof InsertBinaryDataOperation_1.InsertBinaryDataOperation)
                    patch.ops.push(new InsertBinaryDataOperation_1.InsertBinaryDataOperation(ts(op.id), ts(op.obj), ts(op.after), op.data));
                else if (op instanceof MakeArrayOperation_1.MakeArrayOperation)
                    patch.ops.push(new MakeArrayOperation_1.MakeArrayOperation(ts(op.id)));
                else if (op instanceof MakeConstantOperation_1.MakeConstantOperation)
                    patch.ops.push(new MakeConstantOperation_1.MakeConstantOperation(ts(op.id), op.value));
                else if (op instanceof MakeValueOperation_1.MakeValueOperation)
                    patch.ops.push(new MakeValueOperation_1.MakeValueOperation(ts(op.id), op.value));
                else if (op instanceof MakeNumberOperation_1.MakeNumberOperation)
                    patch.ops.push(new MakeNumberOperation_1.MakeNumberOperation(ts(op.id)));
                else if (op instanceof MakeObjectOperation_1.MakeObjectOperation)
                    patch.ops.push(new MakeObjectOperation_1.MakeObjectOperation(ts(op.id)));
                else if (op instanceof MakeStringOperation_1.MakeStringOperation)
                    patch.ops.push(new MakeStringOperation_1.MakeStringOperation(ts(op.id)));
                else if (op instanceof MakeBinaryOperation_1.MakeBinaryOperation)
                    patch.ops.push(new MakeBinaryOperation_1.MakeBinaryOperation(ts(op.id)));
                else if (op instanceof SetNumberOperation_1.SetNumberOperation)
                    patch.ops.push(new SetNumberOperation_1.SetNumberOperation(ts(op.id), ts(op.num), op.value));
                else if (op instanceof SetObjectKeysOperation_1.SetObjectKeysOperation)
                    patch.ops.push(new SetObjectKeysOperation_1.SetObjectKeysOperation(ts(op.id), ts(op.object), op.tuples.map(function (_a) {
                        var _b = (0, tslib_1.__read)(_a, 2), key = _b[0], value = _b[1];
                        return [key, ts(value)];
                    })));
                else if (op instanceof SetRootOperation_1.SetRootOperation)
                    patch.ops.push(new SetRootOperation_1.SetRootOperation(ts(op.id), ts(op.value)));
                else if (op instanceof NoopOperation_1.NoopOperation)
                    patch.ops.push(new NoopOperation_1.NoopOperation(ts(op.id), op.length));
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (ops_1_1 && !ops_1_1.done && (_a = ops_1.return)) _a.call(ops_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return patch;
    };
    return Draft;
}());
exports.Draft = Draft;
