"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogicalDecoder = void 0;
var ArrayChunk_1 = require("../../types/rga-array/ArrayChunk");
var ArrayType_1 = require("../../types/rga-array/ArrayType");
var BinaryChunk_1 = require("../../types/rga-binary/BinaryChunk");
var BinaryType_1 = require("../../types/rga-binary/BinaryType");
var ClockDecoder_1 = require("../../../json-crdt-patch/codec/clock/ClockDecoder");
var ConstantType_1 = require("../../types/const/ConstantType");
var DocRootType_1 = require("../../types/lww-doc-root/DocRootType");
var constants_1 = require("../../constants");
var decode_1 = require("../../../util/base64/decode");
var model_1 = require("../../model");
var ObjectChunk_1 = require("../../types/lww-object/ObjectChunk");
var ObjectType_1 = require("../../types/lww-object/ObjectType");
var constants_2 = require("../../../json-crdt-patch/constants");
var StringChunk_1 = require("../../types/rga-string/StringChunk");
var StringType_1 = require("../../types/rga-string/StringType");
var ValueType_1 = require("../../types/lww-value/ValueType");
var LogicalDecoder = (function () {
    function LogicalDecoder() {
    }
    LogicalDecoder.prototype.decode = function (data) {
        this.clockDecoder = ClockDecoder_1.ClockDecoder.fromArr(data[0]);
        var doc = model_1.Model.withLogicalClock(this.clockDecoder.clock);
        var rootId = this.ts(data, 1);
        var rootNode = data[3] ? this.decodeNode(doc, data[3]) : null;
        doc.root = new DocRootType_1.DocRootType(doc, rootId, rootNode);
        return doc;
    };
    LogicalDecoder.prototype.ts = function (arr, index) {
        var sessionIndex = arr[index];
        var timeDiff = arr[index + 1];
        return this.clockDecoder.decodeId(sessionIndex, timeDiff);
    };
    LogicalDecoder.prototype.decodeNode = function (doc, data) {
        switch (data) {
            case 1:
                return constants_1.NULL;
            case 2:
                return constants_1.FALSE;
            case 3:
                return constants_1.TRUE;
            case 4:
                return constants_1.UNDEFINED;
        }
        if (data instanceof Array) {
            switch (data[0]) {
                case 0:
                    return this.decodeObj(doc, data);
                case 1:
                    return this.decodeArr(doc, data);
                case 2:
                    return this.decodeStr(doc, data);
                case 3:
                    return this.decodeVal(doc, data);
                case 4:
                    return this.decodeConst(doc, data);
                case 5:
                    return this.decodeBin(doc, data);
            }
        }
        throw new Error('UNKNOWN_NODE');
    };
    LogicalDecoder.prototype.decodeObj = function (doc, data) {
        var id = this.ts(data, 1);
        var obj = new ObjectType_1.ObjectType(doc, id);
        var length = data.length;
        for (var i = 3; i < length; i += 4) {
            var key = data[i];
            var keyId = this.ts(data, i + 1);
            var node = this.decodeNode(doc, data[i + 3]);
            var chunk = new ObjectChunk_1.ObjectChunk(keyId, node);
            obj.putChunk(key, chunk);
        }
        doc.nodes.index(obj);
        return obj;
    };
    LogicalDecoder.prototype.decodeArr = function (doc, data) {
        var _this = this;
        var id = this.ts(data, 1);
        var obj = new ArrayType_1.ArrayType(doc, id);
        var length = data.length;
        for (var i = 3; i < length; i += 3) {
            var chunkId = this.ts(data, i);
            var content = data[i + 2];
            var chunk = void 0;
            if (typeof content === 'number') {
                chunk = new ArrayChunk_1.ArrayChunk(chunkId, undefined);
                chunk.deleted = content;
            }
            else {
                var nodes = content.map(function (c) { return _this.decodeNode(doc, c); });
                chunk = new ArrayChunk_1.ArrayChunk(chunkId, nodes);
            }
            obj.append(chunk);
        }
        doc.nodes.index(obj);
        return obj;
    };
    LogicalDecoder.prototype.decodeStr = function (doc, data) {
        var id = this.ts(data, 1);
        var obj = new StringType_1.StringType(doc, id);
        var length = data.length;
        for (var i = 3; i < length; i += 3) {
            var chunkId = this.ts(data, i);
            var content = data[i + 2];
            var chunk = void 0;
            if (typeof content === 'number') {
                chunk = new StringChunk_1.StringChunk(chunkId, undefined);
                chunk.deleted = content;
            }
            else {
                chunk = new StringChunk_1.StringChunk(chunkId, content);
            }
            obj.append(chunk);
        }
        doc.nodes.index(obj);
        return obj;
    };
    LogicalDecoder.prototype.decodeBin = function (doc, data) {
        var id = this.ts(data, 1);
        var obj = new BinaryType_1.BinaryType(doc, id);
        var length = data.length;
        for (var i = 3; i < length; i += 3) {
            var chunkId = this.ts(data, i);
            var content = data[i + 2];
            var chunk = void 0;
            if (typeof content === 'number') {
                chunk = new BinaryChunk_1.BinaryChunk(chunkId, undefined);
                chunk.deleted = content;
            }
            else {
                chunk = new BinaryChunk_1.BinaryChunk(chunkId, (0, decode_1.fromBase64)(content));
            }
            obj.append(chunk);
        }
        doc.nodes.index(obj);
        return obj;
    };
    LogicalDecoder.prototype.decodeVal = function (doc, data) {
        var id = this.ts(data, 1);
        var writeId = this.ts(data, 3);
        var value = data[5];
        var obj = new ValueType_1.ValueType(id, writeId, value);
        doc.nodes.index(obj);
        return obj;
    };
    LogicalDecoder.prototype.decodeConst = function (doc, data) {
        var value = data[1];
        var obj = new ConstantType_1.ConstantType(constants_2.ORIGIN, value);
        return obj;
    };
    return LogicalDecoder;
}());
exports.LogicalDecoder = LogicalDecoder;
