"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelApi = void 0;
var tslib_1 = require("tslib");
var Finder_1 = require("./Finder");
var clock_1 = require("../../../json-crdt-patch/clock");
var NoopOperation_1 = require("../../../json-crdt-patch/operations/NoopOperation");
var constants_1 = require("../../constants");
var Patch_1 = require("../../../json-crdt-patch/Patch");
var PatchBuilder_1 = require("../../../json-crdt-patch/PatchBuilder");
var ModelApi = (function () {
    function ModelApi(model) {
        this.model = model;
        this.patches = [];
        this.builder = new PatchBuilder_1.PatchBuilder(model.clock);
    }
    ModelApi.prototype.getApi = function () {
        return this;
    };
    ModelApi.prototype.getNode = function () {
        var model = this.model;
        var id = model.root.toValue();
        return model.node(id) || constants_1.NULL;
    };
    ModelApi.prototype.find = function (path) {
        var finder = new Finder_1.Finder(this.getNode(), this);
        return path ? finder.find(path) : finder;
    };
    ModelApi.prototype.val = function (path) {
        return this.find().val(path);
    };
    ModelApi.prototype.str = function (path) {
        return this.find().str(path);
    };
    ModelApi.prototype.bin = function (path) {
        return this.find().bin(path);
    };
    ModelApi.prototype.arr = function (path) {
        return this.find().arr(path);
    };
    ModelApi.prototype.obj = function (path) {
        return this.find().obj(path);
    };
    ModelApi.prototype.commit = function () {
        var patch = this.builder.patch;
        this.builder = new PatchBuilder_1.PatchBuilder(this.model.clock);
        if (patch.ops.length) {
            this.patches.push(patch);
            this.model.applyPatch(patch);
        }
        return patch;
    };
    ModelApi.prototype.flush = function () {
        var patches = this.patches;
        this.patches = [];
        return patches;
    };
    ModelApi.prototype.flushPatch = function () {
        var _a, _b;
        var patches = this.flush();
        var length = patches.length;
        var result = new Patch_1.Patch();
        var prev = null;
        for (var i = 0; i < length; i++) {
            var patch = patches[i];
            if (!patch.ops)
                continue;
            if (!prev)
                (_a = result.ops).push.apply(_a, (0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(patch.ops), false));
            else {
                var id = patch.getId();
                var nextTime = prev.nextTime();
                var gap = id.time - nextTime;
                if (gap > 0)
                    result.ops.push(new NoopOperation_1.NoopOperation(new clock_1.LogicalTimestamp(id.getSessionId(), nextTime), gap));
                (_b = result.ops).push.apply(_b, (0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(patch.ops), false));
            }
            prev = patch;
        }
        return result;
    };
    ModelApi.prototype.patch = function (cb) {
        this.commit();
        cb(this);
        this.commit();
    };
    ModelApi.prototype.root = function (json) {
        var value = this.builder.json(json);
        this.builder.root(value);
        return this;
    };
    ModelApi.prototype.toView = function () {
        return this.getNode().toJson();
    };
    return ModelApi;
}());
exports.ModelApi = ModelApi;
